@@ -8,9 +8,9 @@ import requests
 import responses
 from semantic_version import Version
 
+from .. import __version__
 from ..consts import dandi_instance, known_instances
 from ..exceptions import BadCliVersionError, CliVersionTooOldError
-from .. import __version__
 from ..utils import (
     ensure_datetime,
     ensure_strtime,
@@ -18,10 +18,10 @@ from ..utils import (
     flatten,
     flattened,
     get_instance,
+    get_mime_type,
     get_module_version,
     get_utcnow_datetime,
     is_same_time,
-    name2title,
     on_windows,
     remap_dict,
 )
@@ -343,26 +343,6 @@ def test_server_info():
     assert "services" in data
 
 
-@pytest.mark.parametrize(
-    "name,title",
-    [
-        ("relatedResource", "Related Resource"),
-        ("identifier", "Identifier"),
-        ("wasGeneratedBy", "Was Generated by"),
-        ("sameAs", "Same as"),
-        ("includeInCitation", "Include in Citation"),
-        ("anExtraField", "An Extra Field"),
-        ("propertyID", "Property ID"),
-        ("fieldINeed", "Field I Need"),
-        ("needsADatum", "Needs a Datum"),
-        ("contentUrl", "Content URL"),
-        ("ContactPoint", "Contact Point"),
-    ],
-)
-def test_name2title(name, title):
-    assert name2title(name) == title
-
-
 def test_get_module_version():
     import pynwb
 
@@ -372,3 +352,28 @@ def test_get_module_version():
     assert get_module_version("dandi") == __version__
     assert get_module_version("pynwb") == pynwb.__version__
     assert get_module_version("abracadabra123") is None
+
+
+@pytest.mark.parametrize(
+    "filename,mtype",
+    [
+        ("foo.txt", "text/plain"),
+        ("foo", "application/octet-stream"),
+        ("foo.gz", "application/gzip"),
+        ("foo.tar.gz", "application/gzip"),
+        ("foo.tgz", "application/gzip"),
+        ("foo.taz", "application/gzip"),
+        ("foo.svg.gz", "application/gzip"),
+        ("foo.svgz", "application/gzip"),
+        ("foo.Z", "application/x-compress"),
+        ("foo.tar.Z", "application/x-compress"),
+        ("foo.bz2", "application/x-bzip2"),
+        ("foo.tar.bz2", "application/x-bzip2"),
+        ("foo.tbz2", "application/x-bzip2"),
+        ("foo.xz", "application/x-xz"),
+        ("foo.tar.xz", "application/x-xz"),
+        ("foo.txz", "application/x-xz"),
+    ],
+)
+def test_get_mime_type(filename, mtype):
+    assert get_mime_type(filename) == mtype
